@@ -48,6 +48,7 @@
 #'   \item \code{PPIACO} -- Producer Price Index All Commodities
 #'   \item \code{CIVPART} -- Labor Force Participation Rate
 #'   \item \code{M2NS} -- M2 Money Supply
+#'   \item \code{ADPMNUSNERNSA} -- ADP Payrolls
 #'  }
 #' 
 #' @return Returns the following matrices of forecasted variables:
@@ -123,7 +124,7 @@ NNS.nowcast <- function(h = 1,
                  "PCEPI", "PERMIT", "TCU", "BUSINV", "ULCNFB",
                  "IQ", "GACDISA066MSFRBNY", "GACDFSA066MSFRBPHI", "PCEC96", "GDPC1",
                  "ICSA",
-                  "DGS10", "T10Y2Y", "WALCL", "PALLFNFINDEXM", "FEDFUNDS", "PPIACO", "CIVPART", "M2NS")
+                  "DGS10", "T10Y2Y", "WALCL", "PALLFNFINDEXM", "FEDFUNDS", "PPIACO", "CIVPART", "M2NS", "ADPMNUSNERNSA")
 
   sources <- c(rep("FRED", length(variables)), additional.sources)
   
